"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const lodash_filter_1 = tslib_1.__importDefault(require("lodash.filter"));
const query_1 = require("../query");
/**
 * Base class to house relations loading.
 * @internal
 */
class RelationQueryService {
    async queryRelations(RelationClass, relationName, dto, query) {
        if (Array.isArray(dto)) {
            return this.batchQueryRelations(RelationClass, relationName, dto, query);
        }
        const assembler = this.assemblerService.getAssembler(RelationClass, this.getRelationEntity(relationName));
        const relationQueryBuilder = this.getRelationQueryBuilder(relationName);
        return assembler.convertAsyncToDTOs(relationQueryBuilder.select(this.assembler.convertToEntity(dto), assembler.convertQuery(query)).getMany());
    }
    async findRelation(RelationClass, relationName, dto) {
        if (Array.isArray(dto)) {
            return this.batchFindRelations(RelationClass, relationName, dto);
        }
        const assembler = this.assemblerService.getAssembler(RelationClass, this.getRelationEntity(relationName));
        const relationEntity = await this.createRelationQueryBuilder(this.assembler.convertToEntity(dto), relationName).loadOne();
        return relationEntity ? assembler.convertToDTO(relationEntity) : undefined;
    }
    /**
     * Add a single relation.
     * @param id - The id of the entity to add the relation to.
     * @param relationName - The name of the relation to query for.
     * @param relationIds - The ids of relations to add.
     */
    async addRelations(relationName, id, relationIds) {
        const entity = await this.repo.findOneOrFail(id);
        await this.createRelationQueryBuilder(entity, relationName).add(relationIds);
        return this.assembler.convertToDTO(entity);
    }
    /**
     * Set the relation on the entity.
     *
     * @param id - The id of the entity to set the relation on.
     * @param relationName - The name of the relation to query for.
     * @param relationId - The id of the relation to set on the entity.
     */
    async setRelation(relationName, id, relationId) {
        const entity = await this.repo.findOneOrFail(id);
        await this.createRelationQueryBuilder(entity, relationName).set(relationId);
        return this.assembler.convertToDTO(entity);
    }
    /**
     * Removes multiple relations.
     * @param id - The id of the entity to add the relation to.
     * @param relationName - The name of the relation to query for.
     * @param relationIds - The ids of the relations to add.
     */
    async removeRelations(id, relationName, relationIds) {
        const entity = await this.repo.findOneOrFail(id);
        await this.createRelationQueryBuilder(entity, relationName).remove(relationIds);
        return this.assembler.convertToDTO(entity);
    }
    /**
     * Remove the relation on the entity.
     *
     * @param id - The id of the entity to set the relation on.
     * @param relationName - The name of the relation to query for.
     * @param relationId - The id of the relation to set on the entity.
     */
    async removeRelation(id, relationName, relationId) {
        const entity = await this.repo.findOneOrFail(id);
        await this.createRelationQueryBuilder(entity, relationName).remove(relationId);
        return this.assembler.convertToDTO(entity);
    }
    getRelationQueryBuilder(name) {
        return new query_1.RelationQueryBuilder(this.repo, name);
    }
    /**
     * Query for an array of relations for multiple dtos.
     * @param RelationClass - The class to serialize the relations into.
     * @param dto - The dto to query relations for.
     * @param relationName - The name of relation to query for.
     * @param query - A query to filter, page or sort relations.
     */
    async batchQueryRelations(RelationClass, relationName, dtos, query) {
        const assembler = this.assemblerService.getAssembler(RelationClass, this.getRelationEntity(relationName));
        const entities = this.assembler.convertToEntities(dtos);
        const relationQueryBuilder = this.getRelationQueryBuilder(relationName);
        const relations = await relationQueryBuilder.select(entities, assembler.convertQuery(query)).getRawAndEntities();
        return relations.raw.reduce((results, rawRelation) => {
            this.getEntityFromFromResult(rawRelation, entities).forEach(e => {
                const dto = dtos[entities.indexOf(e)];
                if (!results.has(dto)) {
                    results.set(dto, []);
                }
                const relationDtos = assembler.convertToDTOs(this.getRelationsFromPrimaryKeys(relationQueryBuilder, rawRelation, relations.entities));
                results.get(dto).push(...relationDtos);
            });
            return results;
        }, new Map());
    }
    /**
     * Query for a relation for multiple dtos.
     * @param RelationClass - The class to serialize the relations into.
     * @param dtos - The dto to query relations for.
     * @param relationName - The name of relation to query for.
     * @param query - A query to filter, page or sort relations.
     */
    async batchFindRelations(RelationClass, relationName, dtos) {
        const batchResults = await this.batchQueryRelations(RelationClass, relationName, dtos, { paging: { limit: 1 } });
        const results = new Map();
        batchResults.forEach((relation, dto) => {
            // get just the first one.
            results.set(dto, relation[0]);
        });
        return results;
    }
    createRelationQueryBuilder(entity, relationName) {
        return this.repo
            .createQueryBuilder()
            .relation(relationName)
            .of(entity);
    }
    getRelationEntity(relationName) {
        const relationMeta = this.repo.metadata.relations.find(r => r.propertyName === relationName);
        if (!relationMeta) {
            throw new Error(`Unable to find relation ${relationName} on ${this.EntityClass.name}`);
        }
        return relationMeta.type;
    }
    getEntityFromFromResult(rawResult, entities) {
        const pks = Object.keys(rawResult)
            .filter(key => query_1.RelationQueryBuilder.isEntityIdCol(key))
            .reduce((keys, key) => {
            const entityProp = query_1.RelationQueryBuilder.parseEntityIdFromName(key);
            return { ...keys, [entityProp]: rawResult[key] };
        }, {});
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return lodash_filter_1.default(entities, pks);
    }
    getRelationsFromPrimaryKeys(relationBuilder, rawResult, relations) {
        const pks = relationBuilder.getRelationPrimaryKeysPropertyNameAndColumnsName();
        const filter = pks.reduce((keys, key) => {
            return { ...keys, [key.propertyName]: rawResult[key.columnName] };
        }, {});
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return lodash_filter_1.default(relations, filter);
    }
}
exports.RelationQueryService = RelationQueryService;
//# sourceMappingURL=relation-query.service.js.map